@@ -1,5 +1,5 @@
 /*********************************************************************
- *   Copyright 1993, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *********************************************************************/
 
@@ -151,8 +151,10 @@ nclog(NCLOGDBG,"prefetch: %s",var->ncfullname);
     newconstraint->selections = dceclonelist(urlconstraint->selections);
 
     for(i=0;i<nclistlength(vars);i++) {
-        CDFnode* var = (CDFnode*)nclistget(vars,i);
 	DCEprojection* varprojection;
+        CDFnode* var = (CDFnode*)nclistget(vars,i);
+	/* Ignore invisible vars */
+	if(var->invisible) continue; 
 	/* convert var to a projection */
 	ncstat = dapvar2projection(var,&varprojection);
 	if(ncstat != NC_NOERR) {THROWCHK(ncstat); goto done;}
@@ -245,7 +247,7 @@ buildcachenode(NCDAPCOMMON* nccomm,
     /* create the cache node */
     cachenode = createnccachenode();
     cachenode->isprefetch = isprefetch;
-    cachenode->vars = nclistclone(varlist);
+    cachenode->vars = nclistclone(varlist,0);
     cachenode->datadds = dxdroot;
     /* Give the constraint over to the cachenode */
     cachenode->constraint = constraint;
@@ -381,7 +383,7 @@ iscacheableconstraint(DCEconstraint* con)
     int i;
     if(con == NULL) return 1;
     if(con->selections != NULL && nclistlength(con->selections) > 0)
-	return 0; /* cant deal with selections */
+	return 0; /* can't deal with selections */
     for(i=0;i<nclistlength(con->projections);i++) {
         if(!iscacheableprojection((DCEprojection*)nclistget(con->projections,i)))
 	    return 0;
