@@ -1,9 +1,10 @@
 /*********************************************************************
- *   Copyright 1993, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *********************************************************************/
 
 #include "config.h"
+#include "netcdf.h"
 #ifdef USE_PARALLEL
 #include "netcdf_par.h"
 #endif
@@ -13,6 +14,8 @@
 
 #define CHECK(n) if((n) != NC_NOERR) {return (n);} else {}
 
+static char* indentstr = "  "; /* Used by dumpindent */
+
 static void dumptreer(CDFnode* root, NCbytes* buf, int indent, int visible);
 
 int
@@ -31,10 +34,10 @@ dumpmetadata(int ncid, NChdr** hdrp)
 		  &hdr->ngatts,
 		  &hdr->unlimid);
     CHECK(stat);
-    if(ncdap3debug > 0) {
+#ifdef DEBUG2
         fprintf(stdout,"ncid=%d ngatts=%d ndims=%d nvars=%d unlimid=%d\n",
 		hdr->ncid,hdr->ngatts,hdr->ndims,hdr->nvars,hdr->unlimid);
-    }
+#endif
     hdr->gatts = (NCattribute*)calloc(1,hdr->ngatts*sizeof(NCattribute));
     MEMCHECK(hdr->gatts,NC_ENOMEM);
     if(hdr->ngatts > 0)
@@ -89,7 +92,7 @@ dumpmetadata(int ncid, NChdr** hdrp)
         CHECK(stat);
 	fprintf(stdout,"dim[%d]: name=%s size=%lu\n",
 		i,hdr->dims[i].name,(unsigned long)hdr->dims[i].size);
-    }    
+    }
     hdr->vars = (Var*)malloc(hdr->nvars*sizeof(Var));
     MEMCHECK(hdr->vars,NC_ENOMEM);
     for(i=0;i<hdr->nvars;i++) {
@@ -143,7 +146,7 @@ dumpmetadata(int ncid, NChdr** hdrp)
 	    }
 	    fprintf(stdout,"\n");
 	}
-    }    
+    }
     fflush(stdout);
     return NC_NOERR;
 }
@@ -156,10 +159,10 @@ dumpdata1(nc_type nctype, size_t index, char* data)
 	fprintf(stdout,"'%c' %hhd",data[index],data[index]);
 	break;
     case NC_BYTE:
-	fprintf(stdout,"%hdB",((signed char*)data)[index]);
+	fprintf(stdout,"%hhdB",((signed char*)data)[index]);
 	break;
     case NC_UBYTE:
-	fprintf(stdout,"%huB",((unsigned char*)data)[index]);
+	fprintf(stdout,"%hhuB",((unsigned char*)data)[index]);
 	break;
     case NC_SHORT:
 	fprintf(stdout,"%hdS",((short*)data)[index]);
@@ -200,10 +203,7 @@ char*
 dumpprojections(NClist* projections)
 {
     char* tmp;
-    int v = dceverbose;
-    dceverbose = 1;
     tmp = dcelisttostring(projections,",");
-    dceverbose = v;
     return tmp;
 }
 
@@ -211,10 +211,7 @@ char*
 dumpprojection(DCEprojection* proj)
 {
     char* tmp;
-    int v = dceverbose;
-    dceverbose = 1;
     tmp = dcetostring((DCEnode*)proj);
-    dceverbose = v;
     return tmp;
 }
 
@@ -234,10 +231,7 @@ char*
 dumpconstraint(DCEconstraint* con)
 {
     char* tmp;
-    int v = dceverbose;
-    dceverbose = 1;
     tmp = dcetostring((DCEnode*)con);
-    dceverbose = v;
     return tmp;
 }
 
@@ -271,7 +265,6 @@ dumppath(CDFnode* leaf)
 static void
 dumpindent(int indent, NCbytes* buf)
 {
-    static char* indentstr = "  ";
     int i;
     for(i=0;i<indent;i++) ncbytescat(buf,indentstr);
 }
@@ -346,7 +339,7 @@ dumptreer(CDFnode* root, NCbytes* buf, int indent, int visible)
 	ncbytescat(buf," ");
         ncbytescat(buf,(root->ncbasename?root->ncbasename:"<?>"));
 	break;
-    default: break;    
+    default: break;
     }
 
     if(nclistlength(root->array.dimsetplus) > 0) dimset = root->array.dimsetplus;
@@ -423,12 +416,12 @@ dumpnode(CDFnode* node)
 	default: break;
 	}
 	break;
-    default: break;    
+    default: break;
     }
     snprintf(tmp,sizeof(tmp),"%s %s {\n",
 		(nctype?nctype:primtype),node->ocname);
     ncbytescat(buf,tmp);
-    snprintf(tmp,sizeof(tmp),"ocnode=%lx\n",(unsigned long)node->ocnode);
+    snprintf(tmp,sizeof(tmp),"ocnode=%p\n",node->ocnode);
     ncbytescat(buf,tmp);
     snprintf(tmp,sizeof(tmp),"container=%s\n",
 		(node->container?node->container->ocname:"null"));
@@ -440,7 +433,7 @@ dumpnode(CDFnode* node)
     ncbytescat(buf,tmp);
     snprintf(tmp,sizeof(tmp),"ncfullname=%s\n",node->ncfullname);
     ncbytescat(buf,tmp);
-    snprintf(tmp,sizeof(tmp),"|subnodes|=%ld\n",nclistlength(node->subnodes));
+    snprintf(tmp,sizeof(tmp),"|subnodes|=%u\n",(unsigned)nclistlength(node->subnodes));
     ncbytescat(buf,tmp);
     snprintf(tmp,sizeof(tmp),"externaltype=%d\n",node->externaltype);
     ncbytescat(buf,tmp);
@@ -459,7 +452,7 @@ dumpnode(CDFnode* node)
     snprintf(tmp,sizeof(tmp),"attachment=%s\n",
 		(node->attachment?node->attachment->ocname:"null"));
     ncbytescat(buf,tmp);
-    snprintf(tmp,sizeof(tmp),"rank=%lu\n",nclistlength(node->array.dimset0));
+    snprintf(tmp,sizeof(tmp),"rank=%u\n",(unsigned)nclistlength(node->array.dimset0));
     ncbytescat(buf,tmp);
     for(i=0;i<nclistlength(node->array.dimset0);i++) {
 	CDFnode* dim = (CDFnode*)nclistget(node->array.dimset0,i);
@@ -485,14 +478,14 @@ dumpnode(CDFnode* node)
 }
 
 char*
-dumpalign(NCalignment* ncalign)
+dumpalign(NCD2alignment* ncalign)
 {
     char* result;
     char tmp[1024];
     if(ncalign == NULL)
-	result = nulldup("NCalignment{size=-- alignment=-- offset=--}");
+	result = nulldup("NCD2alignment{size=-- alignment=-- offset=--}");
     else {
-        snprintf(tmp,sizeof(tmp),"NCalignment{size=%lu alignment=%lu offset=%lu}",
+        snprintf(tmp,sizeof(tmp),"NCD2alignment{size=%lu alignment=%lu offset=%lu}",
 		 ncalign->size,ncalign->alignment,ncalign->offset);
         result = nulldup(tmp);
     }
@@ -510,9 +503,9 @@ dumpcachenode(NCcachenode* node)
     if(node == NULL) return strdup("cachenode{null}");
     buf = ncbytesnew();
     result = dcebuildconstraintstring(node->constraint);
-    snprintf(tmp,sizeof(tmp),"cachenode%s(%lx){size=%lu; constraint=%s; vars=",
+    snprintf(tmp,sizeof(tmp),"cachenode%s(%p){size=%lu; constraint=%s; vars=",
 		node->isprefetch?"*":"",
-		(unsigned long)node,
+		node,
 		(unsigned long)node->xdrsize,
 	        result);
     ncbytescat(buf,tmp);
