@@ -1,5 +1,5 @@
 /*
-  Copyright 2004-2006, UCAR/Unidata
+  Copyright 2004-2018, UCAR/Unidata
   See COPYRIGHT file for copying and redistribution conditions.
 
   This is part of netCDF.
@@ -11,7 +11,7 @@
   This program is an add-on test to check very large 64-bit offset
   files (8 GB, so make sure you have the disk space!).
 
-  $Id: large_files.c,v 1.4 2008/03/10 17:02:41 ed Exp $
+  Ed Hartnett, Ward Fisher
 */
 
 #include <config.h>
@@ -21,6 +21,11 @@
 #include <netcdf.h>
 
 #define FILE_NAME "large_files.nc"
+#define NUMRECS 2
+#define I_LEN 4106
+#define J_LEN 1023
+#define K_LEN 1023
+#define N_LEN 2
 
 static void
 check_err(const int stat, const int line, const char *file) {
@@ -36,7 +41,7 @@ main(int argc, char **argv) {
    int  stat;			/* return status */
    char file_name[NC_MAX_NAME + 1];
    int  ncid;			/* netCDF id */
-   int rec, i, j, k;
+   int rec, i, j;
    int x[] = {42, 21};
 
    /* dimension ids */
@@ -46,17 +51,6 @@ main(int argc, char **argv) {
    int k_dim;
    int n_dim;
 
-#define NUMRECS 2
-#define I_LEN 4106
-   //#define I_LEN 5
-   //#if 0
-   //#define J_LEN 214700000
-   //#endif
-   //#define J_LEN   500000000
-#define J_LEN 1023
-#define K_LEN 1023
-#define N_LEN 2
-
    /* dimension lengths */
    size_t rec_len = NC_UNLIMITED;
    size_t i_len = I_LEN;
