@@ -50,10 +50,10 @@ main(int argc, char **argv)
       yscaleDims[0] = ncolCur;
       if ((xdimSpaceId = H5Screate_simple(1, xscaleDims, NULL)) < 0) ERR;
 
-      /* With the SEMI close degree, the HDF5 file close will fail if
+      /* With the WEAK close degree, the HDF5 file close will not fail if
        * anything is left open. */
       if ((fapl = H5Pcreate(H5P_FILE_ACCESS)) < 0) ERR;
-      if (H5Pset_fclose_degree(fapl, H5F_CLOSE_SEMI)) ERR;
+      if (H5Pset_fclose_degree(fapl, H5F_CLOSE_WEAK)) ERR;
 
       /* Create file */
       if((fileId = H5Fcreate(FILE_NAME, H5F_ACC_TRUNC,
@@ -180,7 +180,7 @@ main(int argc, char **argv)
       }
    }
    SUMMARIZE_ERR;
-#ifdef USE_SZIP
+#ifdef HAVE_H5Z_SZIP
    printf("*** testing HDF5 compatibility with szip...");
    {
 
@@ -211,7 +211,7 @@ main(int argc, char **argv)
       /* Open file and create group. */
       if ((fileid = H5Fcreate(FILE_NAME, H5F_ACC_TRUNC, H5P_DEFAULT,
 			      H5P_DEFAULT)) < 0) ERR;
-      if ((grpid = H5Gcreate(fileid, GRP_NAME, 0)) < 0) ERR;
+      if ((grpid = H5Gcreate1(fileid, GRP_NAME, 0)) < 0) ERR;
 
       /* Write an array of bools, with szip compression. */
       if ((propid = H5Pcreate(H5P_DATASET_CREATE)) < 0) ERR;
@@ -219,7 +219,7 @@ main(int argc, char **argv)
       if (H5Pset_chunk(propid, 1, dims)) ERR;
       if (H5Pset_szip(propid, H5_SZIP_EC_OPTION_MASK, 32)) ERR;
       if ((spaceid = H5Screate_simple(1, dims, dims)) < 0) ERR;
-      if ((datasetid = H5Dcreate(grpid, BATTLE_RECORD, H5T_NATIVE_INT,
+      if ((datasetid = H5Dcreate1(grpid, BATTLE_RECORD, H5T_NATIVE_INT,
 				 spaceid, propid)) < 0) ERR;
       if (H5Dwrite(datasetid, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT,
 		   data_out) < 0) ERR;
@@ -254,7 +254,7 @@ main(int argc, char **argv)
 
    }
    SUMMARIZE_ERR;
-#endif /* USE_SZIP */
+#endif /* HAVE_H5Z_SZIP */
    /* This test suggested by user brentd42 to find a memory problem in
     * function rec_read_metadata(). This test demonstrates the bug on
     * address sanitizer runs. See
