@@ -1,9 +1,8 @@
 #!/bin/sh
 
-if test "x$SETX" = x1 ; then set -x ; fi
-
 if test "x$srcdir" = x ; then srcdir=`pwd`; fi
 . ../test_common.sh
+
 set -e
 
 . ${srcdir}/tst_utils.sh
@@ -34,7 +33,7 @@ EXPECTED="$expected3"
 RESULTSDIR="file_results"
 
 rm -fr ${RESULTSDIR}
-mkdir "${RESULTSDIR}"
+mkdir -p "${RESULTSDIR}"
 
 echo "*** Testing $TITLE "
 echo "        Base URL: ${TESTURL}"
