@@ -1,6 +1,6 @@
 #!/bin/sh
 # This shell script runs the ncdump tests.
-# $Id: run_tests.sh,v 1.18 2010/05/19 13:43:39 ed Exp $
+# Ed Hartnett, Dennis Heimbigner
 
 if test "x$srcdir" = x ; then srcdir=`pwd`; fi
 . ../test_common.sh
@@ -9,7 +9,6 @@ set -e
 echo ""
 echo "*** Testing ncgen and ncdump using some test CDL files."
 echo "*** creating tst_small.nc from ref_tst_small.cdl..."
-cat $srcdir/ref_tst_small.cdl
 ${NCGEN} -b -o tst_small.nc $srcdir/ref_tst_small.cdl
 echo "*** creating tst_small.cdl from tst_small.nc..."
 ${NCDUMP} tst_small.nc > tst_small.cdl
