@@ -1,11 +1,10 @@
 /*********************************************************************
- *   Copyright 1993, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *   $Header: /upc/share/CVS/netcdf-3/ncgen/genf77.c,v 1.4 2010/05/17 23:26:44 dmh Exp $
  *********************************************************************/
 
 #include "includes.h"
-#include "nc_iter.h"
 
 #ifdef ENABLE_F77
 
@@ -571,43 +570,6 @@ f77attrifyr(Symbol* asym, char* p, Bytebuffer* buf)
     return p;
 }
 
-#ifdef USE_NETCDF4
-#if 0
-/* Result is pool alloc'd*/
-static char*
-f77prefixed(List* prefix, char* suffix, char* separator)
-{
-    int slen;
-    int plen;
-    int i;
-    char* result;
-
-    ASSERT(suffix != NULL);
-    plen = prefixlen(prefix);
-    if(prefix == NULL || plen == 0) return codify(suffix);
-    /* plen > 0*/
-    slen = 0;
-    for(i=0;i<plen;i++) {
-	Symbol* sym = (Symbol*)listget(prefix,i);
-	slen += (strlen(sym->name)+strlen(separator));
-    }
-    slen += strlen(suffix);
-    slen++; /* for null terminator*/
-    result = poolalloc(slen);
-    result[0] = '\0';
-    /* Leave off the root*/
-    i = (rootgroup == (Symbol*)listget(prefix,0))?1:0;
-    for(;i<plen;i++) {
-	Symbol* sym = (Symbol*)listget(prefix,i);
-        strcat(result,sym->name); /* append "<prefix[i]/>"*/
-	strcat(result,separator);
-    }    
-    strcat(result,suffix); /* append "<suffix>"*/
-    return result;
-}
-#endif
-#endif
-
 /* return FORTRAN name for netCDF type, given type code */
 static const char*
 nftype(nc_type type)
