@@ -1,5 +1,5 @@
 /*********************************************************************
- *   Copyright 1993, UCAR/Unidata
+ *   Copyright 2018, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *   $Header: /upc/share/CVS/netcdf-3/ncgen/getfill.c,v 1.8 2010/04/14 22:04:56 dmh Exp $
  *********************************************************************/
@@ -15,70 +15,96 @@ static void fill(Symbol* tsym, Datalist*);
 static void fillarray(Symbol* tsym, Dimset* dimset, int index, Datalist*);
 static void filllist(Symbol* tvsym, Datalist* dl);
 
-/* Construct a Datalist representing a complete fill value*/
-/* for a specified variable */
-/* Cache if needed later*/
+/* Construct a Datalist representing a complete fill value.
+   for a specified variable or type. Cache if needed later.
+   The rules are as follows
+   1. If the tvsym argument is a variable and it has a _FillValue
+      attribute, then use that value.
+   2. If the tvsym argunment is a variable with no specified
+      _FillValue, then create one based on the variable's basetype.
+   3. If the tvsym argument is a type, then if it already has a fill value,
+      then use that, otherwise build and cache a fillvalue appropriate
+      for that type.
+*/
 
 Datalist*
 getfiller(Symbol* tvsym)
 {
     Datalist* filler = NULL;
-    Symbol* tsym = tvsym;
     ASSERT(tvsym->objectclass == NC_VAR || tvsym->objectclass == NC_TYPE);
     if(tvsym->objectclass == NC_VAR) {
-	tsym = tvsym->typ.basetype;
-        filler = tvsym->var.special._Fillvalue; /* get cached value (if any)*/
-    }
-    if(filler == NULL || tvsym->objectclass == NC_TYPE) {
-        filler = tvsym->typ._Fillvalue; /* get cached value (if any)*/
-    }
-    if(filler == NULL) { /* need to compute it*/
-        filler = builddatalist(0);
-	fill(tsym,filler);
+        if(tvsym->var.special._Fillvalue !=  NULL) {
+	    /* We have a _FillValue Attribute specified */
+	    filler = tvsym->var.special._Fillvalue;
+        } else { /* otherwise create a fillvalue for the base type */
+	    filler = getfiller(tvsym->typ.basetype);
+	}
+    } else { /* (tvsym->objectclass == NC_TYPE) */
+        if(tvsym->typ._Fillvalue ==  NULL) {
+	    /* create and cache */
+            filler = builddatalist(0);
+	    fill(tvsym,filler);
+	    tvsym->typ._Fillvalue = filler;
+	}
+	filler = tvsym->typ._Fillvalue;
     }
 #ifdef GENDEBUG2
     dumpdatalist(filler,"getfiller");
 #endif
-    if(tvsym->objectclass == NC_VAR) {
-        tvsym->var.special._Fillvalue = filler;
-    } else if(tvsym->objectclass == NC_TYPE) {
-        tvsym->typ._Fillvalue = filler; /* cache value*/
-    }
-    return filler;        
+    return filler;
 }
 
 static void
 fill(Symbol* tsym, Datalist* filler)
 {
     unsigned long i;
-    NCConstant con = nullconstant;
-    Datalist* sublist;
+    NCConstant* con = NULL;
 
     ASSERT(tsym->objectclass == NC_TYPE);
     switch (tsym->subclass) {
     case NC_ENUM: case NC_OPAQUE: case NC_PRIM:
-        con.nctype = tsym->typ.typecode;
-        nc_getfill(&con);
+        con = nullconst();
+        con->nctype = tsym->typ.typecode;
+        nc_getfill(con,tsym);
+        dlappend(filler,con);
 	break;
-    case NC_COMPOUND:
-	sublist = builddatalist(listlength(tsym->subnodes));
+    case NC_COMPOUND: {
+	/* Given a compound type, the fill will be a sublist
+	   consisting itself of N constants, where N is the number of fields.
+	   Non-array fields will be direct, array fields will be sublists.
+	*/
+	Datalist* cmpdlist = builddatalist(listlength(tsym->subnodes)); /* list of field constants */
         for(i=0;i<listlength(tsym->subnodes);i++) {
+	    NCConstant* fieldinstance = NULL;
 	    Symbol* field = (Symbol*)listget(tsym->subnodes,i);
 	    if(field->typ.dimset.ndims > 0) {	
-                fillarray(field->typ.basetype,&field->typ.dimset,0,filler);
+  	        /*  Build a sublist for this field */
+	        Datalist* arraydata = builddatalist(0);
+                fillarray(field->typ.basetype,&field->typ.dimset,0,arraydata);
+		/* Convert to a compound constant */
+	        fieldinstance = list2const(arraydata);
+		dlappend(cmpdlist,fieldinstance);
+		fieldinstance = NULL;
 	    } else
-		filllist(field->typ.basetype,sublist);
+		/* Append directly to cmpdlist*/
+		filllist(field->typ.basetype,cmpdlist);
         }	  
-	con = builddatasublist(sublist);
-	break;
-    case NC_VLEN:
-	sublist = builddatalist(0);
-	filllist(tsym->typ.basetype,sublist); /* generate a single instance*/
-	con = builddatasublist(sublist);
-	break;
+        /* Add compound instance to the filler list */
+	con = list2const(cmpdlist);
+        dlappend(filler,con);
+	con = NULL;
+	} break;
+    case NC_VLEN: {
+        Datalist* vlensublist = NULL;
+	/* The vlist will have a sublist hanging off of fille, with
+	   a single compound constant in filler */
+	vlensublist = builddatalist(0);
+	filllist(tsym->typ.basetype,vlensublist); /* generate a single instance*/
+	con = builddatasublist(vlensublist);
+        dlappend(filler,con);
+	} break;
     default: PANIC1("fill: unexpected subclass %d",tsym->subclass);
     }
-    dlappend(filler,&con);
 }
 
 static void
@@ -86,14 +112,15 @@ filllist(Symbol* tsym, Datalist* dl)
 {
     int i;
     Datalist* sublist;
-    NCConstant con = nullconstant;
+    NCConstant* con = NULL;
 
     ASSERT(tsym->objectclass == NC_TYPE);
     switch (tsym->subclass) {
     case NC_ENUM: case NC_OPAQUE: case NC_PRIM:
-        con.nctype = tsym->typ.typecode;
-        nc_getfill(&con);
-	dlappend(dl,&con);
+        con = nullconst();
+        con->nctype = tsym->typ.typecode;
+        nc_getfill(con,tsym);
+	dlappend(dl,con);
 	break;
     case NC_COMPOUND:
 	sublist = builddatalist(listlength(tsym->subnodes));
@@ -102,18 +129,21 @@ filllist(Symbol* tsym, Datalist* dl)
 	    filllist(field->typ.basetype,sublist);
         }	  
 	con = builddatasublist(sublist);
-	dlappend(dl,&con);
+	dlappend(dl,con);
 	break;
     case NC_VLEN:
 	sublist = builddatalist(0);
 	filllist(tsym->typ.basetype,sublist); /* generate a single instance*/
 	con = builddatasublist(sublist);
-	dlappend(dl,&con);
+	dlappend(dl,con);
 	break;
     default: PANIC1("fill: unexpected subclass %d",tsym->subclass);
     }
 }
 
+/* Create an array of fill values of basetype given the dimset. This
+is recursive over the dimensions as specified by the index argumen
+*/
 static void
 fillarray(Symbol* basetype, Dimset* dimset, int index, Datalist* arraylist)
 {
@@ -122,23 +152,20 @@ fillarray(Symbol* basetype, Dimset* dimset, int index, Datalist* arraylist)
     unsigned int size = dim->dim.declsize;
     int isunlimited = (size == 0);
     int lastdim = (index == (dimset->ndims - 1));
-    int firstdim = (index == 0);
-    Datalist* sublist;
 
-    sublist = (firstdim?builddatalist(0):arraylist);
     if(isunlimited) {
 	/* do a single entry to satisfy*/
         if(lastdim) {
-	    filllist(basetype,sublist);
+	    filllist(basetype,arraylist);
 	} else {
-	    fillarray(basetype->typ.basetype,dimset,index+1,sublist);
+	    fillarray(basetype->typ.basetype,dimset,index+1,arraylist);
 	}
     } else { /* bounded*/
         if(lastdim) {
-	    for(i=0;i<size;i++) filllist(basetype,sublist);
+	    for(i=0;i<size;i++) filllist(basetype,arraylist);
 	} else {
 	    for(i=0;i<size;i++) {
-	        fillarray(basetype->typ.basetype,dimset,index+1,sublist);
+	        fillarray(basetype->typ.basetype,dimset,index+1,arraylist);
 	    }
 	}
     }
@@ -149,7 +176,7 @@ fillarray(Symbol* basetype, Dimset* dimset, int index, Datalist* arraylist)
  * that type.
  */
 void
-nc_getfill(NCConstant* value)
+nc_getfill(NCConstant* value, Symbol* tsym)
 {
     switch(value->nctype) {
       case NC_CHAR: value->value.charv = NC_FILL_CHAR; break;
@@ -174,6 +201,33 @@ nc_getfill(NCConstant* value)
         value->value.opaquev.len = 2;
         value->value.opaquev.stringv = nulldup("00");
 	break;
+      case NC_ENUM: {
+	Symbol* econst;
+	NCConstant* eccon;
+	if(tsym == NULL)
+	    derror("nc_getfill: no enum type specified");
+	/* Get the first value */
+	if(tsym->subclass != NC_ENUM)
+	    derror("nc_getfill: expected enum type");
+	if(listlength(tsym->subnodes) == 0)
+	    derror("nc_getfill: empty enum type");
+	econst = listget(tsym->subnodes,0);
+	eccon = econst->typ.econst;
+	switch (eccon->nctype) {
+	case NC_BYTE:
+	case NC_SHORT:
+	case NC_INT:
+	case NC_UBYTE:
+	case NC_USHORT:
+	case NC_UINT:
+	case NC_INT64:
+	case NC_UINT64:
+	    value->value = eccon->value;
+	    break;
+	default:
+	    derror("nc_getfill: illegal enum basetype");
+	}
+      } break;
       default:
 	derror("nc_getfill: unrecognized type: %d",value->nctype);
     }
