@@ -9,7 +9,11 @@ if test "x$srcdir" = x ;then srcdir=`pwd`; fi
 . ../test_common.sh
 
 echo "*** creating classic file c0.nc from c0.cdl..."
-${NCGEN3} -b -o c0.nc ${ncgen3c0}
+if ${NCGEN3} -b -o c0.nc ${ncgen3c0} ; then
+echo "***PASS"
+else
+echo "***fail"
+fi
 echo "*** creating 64-bit offset file c0_64.nc from c0.cdl..."
 #${NCGEN3} -k 64-bit-offset -b -o c0_64.nc ${ncgen3c0}
 
