@@ -1,4 +1,4 @@
-/* Copyright 2009, UCAR/Unidata and OPeNDAP, Inc.
+/* Copyright 2018, UCAR/Unidata and OPeNDAP, Inc.
    See the COPYRIGHT file for more information. */
 
 #ifndef OCCONSTRAINTS_H
@@ -15,14 +15,14 @@ typedef struct OCslice {
 
 /*! Specifies a form of path where each element can have a set of associated indices */
 typedef struct OCpath {
-    OClist* names;
-    OClist* indexsets; /* oclist<oclist<Slice>> */
+    NClist* names;
+    NClist* indexsets; /* oclist<oclist<Slice>> */
 } OCpath;
 
 /*! Specifies a ProjectionClause. */
 typedef struct OCprojectionclause {
     char* target; /* "variable name" as mentioned in the projection */
-    OClist* indexsets; /* oclist<oclist<OCslice>> */
+    NClist* indexsets; /* oclist<oclist<OCslice>> */
     struct OCnode* node; /* node with name matching target, if any. */
     int    gridconstraint; /*  used only for testing purposes */
 } OCprojectionclause;
@@ -32,7 +32,7 @@ typedef struct OCselectionclause {
     int op;
     char* value;
     struct OCselectionclause* lhs;
-    OClist* rhs;
+    NClist* rhs;
 } OCselectionclause;
 
 
