@@ -139,7 +139,8 @@ fail:
 	nclog(NCLOGERR, "curl error: %s", curl_easy_strerror(cstat));
 	switch (httpcode) {
 	case 400: stat = OC_EBADURL; break;
-	case 401: stat = OC_EAUTH; break;
+	case 401: stat = OC_EACCESS; break;
+	case 403: stat = OC_EAUTH; break;
 	case 404: stat = OC_ENOFILE; break;
 	case 500: stat = OC_EDAPSVC; break;
 	case 200: break;
@@ -274,7 +275,7 @@ ocfetchlastmodified(CURL* curl, char* url, long* filetime)
 
     /* Ask for head */
     cstat = CURLERR(curl_easy_setopt(curl, CURLOPT_TIMEOUT, 30)); /* 30sec timeout*/
-    cstat = CURLERR(curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 2));
+    cstat = CURLERR(curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 5));
     cstat = CURLERR(curl_easy_setopt(curl, CURLOPT_HEADER, 1));
     cstat = CURLERR(curl_easy_setopt(curl, CURLOPT_NOBODY, 1));
     cstat = CURLERR(curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 1));
@@ -313,6 +314,11 @@ ocping(const char* url)
     if (cstat != CURLE_OK)
         goto done;
 
+    /* use a very short conn timeout: 10 seconds */
+    cstat = CURLERR(curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, (long)10));
+    if (cstat != CURLE_OK)
+        goto done;
+
     /* use a very short timeout: 5 seconds */
     cstat = CURLERR(curl_easy_setopt(curl, CURLOPT_TIMEOUT, (long)5));
     if (cstat != CURLE_OK)
